@@ -17,47 +17,81 @@ js2py_array(Context* cx, jsval val)
 Py_ssize_t
 Array_length(Object* self)
 {
+    Py_ssize_t ret = -1;
     jsuint length;
+
+    JS_BeginRequest(self->cx->cx);
+
     if(!JS_GetArrayLength(self->cx->cx, self->obj, &length))
     {
         PyErr_SetString(PyExc_AttributeError, "Failed to get array length.");
-        return -1;
+        goto done;
     }
 
-    return (Py_ssize_t) length;
+     ret = (Py_ssize_t) length;
+    
+done:
+    JS_EndRequest(self->cx->cx);
+    return ret;
 }
 
 PyObject*
 Array_get_item(Object* self, Py_ssize_t idx)
 {
+    PyObject* ret = NULL;
     jsval rval;
     jsint pos = (jsint) idx;
 
+    JS_BeginRequest(self->cx->cx);
+
+    if(idx >= Array_length(self))
+    {
+        PyErr_SetString(PyExc_IndexError, "List index out of range.");
+        goto done;
+    }
+
     if(!JS_GetElement(self->cx->cx, self->obj, pos, &rval))
     {
         PyErr_SetString(PyExc_AttributeError, "Failed to get array item.");
-        return NULL;
+        goto done;
     }
 
-    return js2py(self->cx, rval);
+    ret = js2py(self->cx, rval);
+
+done:
+    JS_EndRequest(self->cx->cx);
+    return ret;
 }
 
 int
 Array_set_item(Object* self, Py_ssize_t idx, PyObject* val)
 {
+    int ret = -1;
     jsval pval;
     jsint pos = (jsint) idx;
 
+    JS_BeginRequest(self->cx->cx);
+
     pval = py2js(self->cx, val);
-    if(pval == JSVAL_VOID) return -1;
+    if(pval == JSVAL_VOID) goto done;
 
     if(!JS_SetElement(self->cx->cx, self->obj, pos, &pval))
     {
         PyErr_SetString(PyExc_AttributeError, "Failed to set array item.");
-        return -1;
+        goto done;
     }
 
-    return 0;
+    ret = 0;
+
+done:
+    JS_EndRequest(self->cx->cx);
+    return ret;
+}
+
+PyObject*
+Array_iterator(Object* self)
+{
+    return PySeqIter_New((PyObject*) self);
 }
 
 static PyMemberDef Array_members[] = {
@@ -108,7 +142,7 @@ PyTypeObject _ArrayType = {
     0,		                                    /*tp_clear*/
     0,		                                    /*tp_richcompare*/
     0,		                                    /*tp_weaklistoffset*/
-    0,		                                    /*tp_iter*/
+    (getiterfunc)Array_iterator,		        /*tp_iter*/
     0,		                                    /*tp_iternext*/
     Array_methods,                              /*tp_methods*/
     Array_members,                              /*tp_members*/
